/**
  * Aller (le terrier d'AbulEdu)
  *
  * @warning aucun traitement d'erreur n'est pour l'instant implémenté
  * @see https://redmine.ryxeo.com/projects/leterrier-aller
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @author 2011 Gabriel Cadeillan <gabriel.cadeillan@ryxeo.com>
  * @see The GNU Public License (GPL)
  */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <QApplication>
#include "abuledulabel.h"
#include <QDebug>
#include <QPixmap>
#include <QRgb>
#include <pixel.h>

const QString STYLESELECTION = "background-color: #FFDAB9";
const QString STYLEERREUR = "background-color: #FF7F50";

AbuleduLabel::AbuleduLabel(QWidget *parent) :
        QLabel(parent)
{
    //Le constructeur cree les objets forcement en 0x0 et sans nom ...
    //qDebug() << "Constructeur pour l'objet " << this->objectName() << " on se place a " << x() << " x " << y();
    // Pour quoi faire ?
    m_mimecode="abuledulabel/x-abuledu";
    setMargeVerticale(0);
    setFrameShape(QFrame::Box);
    setCursor(Qt::PointingHandCursor);
}

AbuleduLabel::AbuleduLabel(QString texte, int numOrdre, int numPlace, QWidget *parent) :
        QLabel(parent)
{
    m_mimecode="abuledulabel/x-abuledu";
    setObjectName("AbuleduLabel");
    setText(texte);
    setNumOrdre(numOrdre);
    setNumPlace(numPlace);
    setLastPos(QPoint(0,0));
    setMargeVerticale(0);
    setFrameShape(QFrame::Box);
    setCursor(Qt::PointingHandCursor);
    if (parent)
    {
        setMaximumWidth(parent->width());
    }
    setWordWrap(false);

}
void AbuleduLabel::setNumOrdre(int intOrdre)
{
    m_intOrdre=intOrdre;
}

int AbuleduLabel::numOrdre()
{
    return m_intOrdre;
}

void AbuleduLabel::setNumPlace(int intPlace)
{
    m_intPlace=intPlace;
}
int AbuleduLabel::numPlace()
{
    return m_intPlace;
}

void AbuleduLabel::setLastPos(QPoint lastPos)
{
    m_lastPos=lastPos;
}

void AbuleduLabel::setLastPos(int x, int y)
{
    m_lastPos=QPoint(x,y);
}

QPoint AbuleduLabel::lastPos()
{
    return m_lastPos;
}

void AbuleduLabel::setMargeVerticale(int marge)
{
    if(marge>=0)
        m_margeVerticale=marge;
}

void AbuleduLabel::setMargeHorizontale(int marge)
{
    if(marge>=0)
        m_margeHorizontale=marge;
}

int AbuleduLabel::margeHorizontale()
{
    return m_margeHorizontale;
}


/* Filtre qui pixelise le pixmap du AbuleduLabel
   la taille des pixel "grossis" est en parametre taillePixel
   l'algorithme fait la moyenne des valeurs rouge,vert et bleu des pixels de chaques zone taillePixel*taillePixel
   puis applique à tous ces pixel la valeur moyenne*/
void AbuleduLabel::pixelize(int taillePixel)
{
    QImage imageTemp;
    imageTemp = this->pixmap()->toImage();

    int sommeRouge;
    int sommeVert;
    int sommeBleu;
    int nombrePixelMoyenne;

    for(int i=0;i<=imageTemp.width()/taillePixel;i++)
    {
        for(int j=0;j<=imageTemp.height()/taillePixel;j++)
        {
            sommeRouge = 0;
            sommeVert = 0;
            sommeBleu = 0;
            nombrePixelMoyenne = 0;

            for(int k=i*taillePixel;k<(i+1)*taillePixel;k++)
            {
                for(int l=j*taillePixel;l<(j+1)*taillePixel;l++)
                {
                    if(k<imageTemp.width() && l<imageTemp.height())
                    {
                        sommeRouge+=qRed(imageTemp.pixel(k,l));
                        sommeVert+=qGreen(imageTemp.pixel(k,l));
                        sommeBleu+=qBlue(imageTemp.pixel(k,l));
                        nombrePixelMoyenne+=1;
                    }
                }
            }

            if(nombrePixelMoyenne!=0)
            {
                sommeRouge = sommeRouge/(nombrePixelMoyenne);
                sommeVert = sommeVert/(nombrePixelMoyenne);
                sommeBleu = sommeBleu/(nombrePixelMoyenne);
            }

            QRgb valeurMoyennePixel;

            valeurMoyennePixel = qRgb(sommeRouge, sommeVert, sommeBleu);

            for(int k=i*taillePixel;k<(i+1)*taillePixel;k++)
            {
                for(int l=j*taillePixel;l<(j+1)*taillePixel;l++)
                {
                    if(k<imageTemp.width() && l<imageTemp.height())
                    {
                        imageTemp.setPixel(k, l, valeurMoyennePixel);
                    }
                }
            }

        }
    }


    //imageTemp.invertPixels();
    this->setPixmap(QPixmap::fromImage(imageTemp));
}

//-----------------------------------------------------
//
//             Gestion du Drag & Drop
//
//-----------------------------------------------------

void AbuleduLabel::mousePressEvent(QMouseEvent *event)
{
    if(!acceptDrops())return; // Ne fait rien si l'exo n'est pas commencé

    // Déselectionner les étiquettes
    QList<AbuleduLabel *> listeEtiquettes = this->parentWidget()->findChildren<AbuleduLabel *>();
    foreach(AbuleduLabel* etiq, listeEtiquettes)
    {
        etiq->setStyleSheet("");
    }
    this->setStyleSheet(STYLESELECTION);

    // On crée un drag minimum car il ne servira pas, à moins qu'on décide autrement !
    if (!(event->buttons() & Qt::LeftButton))
        return;
    QPoint hotSpot = event->pos();
    QDrag *drag = new QDrag(this);
    QMimeData *mimeData = new QMimeData; // Faudra peut-être un mimetype abuledu
    QByteArray itemData;
    QDataStream dataStream(&itemData, QIODevice::WriteOnly);
    dataStream << m_mimecode ;

    mimeData->setData(m_mimecode, itemData);
    mimeData->setText(this->text());
    drag->setHotSpot(hotSpot);

    drag->setMimeData(mimeData);
    QPixmap pix(this->size());
    this->render(&pix);
    drag->setPixmap(pix);
    this->hide();
    Qt::DropAction dropAction = drag->exec(Qt::MoveAction);
    qDebug()<<"DropAction"<<dropAction;
    this->move(this->lastPos());
    this->show(); // Si l'étiquette est dropée ailleurs, on la réaffiche
    emit abuleduLabelSelected(this->numPlace());
    event->accept();
}

void AbuleduLabel::dragEnterEvent(QDragEnterEvent * event)
{
    //qDebug()<<"event type"<<event->type();
    //
    //if(event->source()->objectName()=="abuleduLabel")
    //if(QString(event->source()->metaObject()->className()) != "AbuleduLabel")// On vérifie que l'on a bien une étiquette
    if(!event->mimeData()->formats().contains(m_mimecode))
    {
        event->ignore();
    }
    else  // On déplace l'étiquette (ou pas)
    {
        //qDebug()<<"Numero de l'etiquette survolee"<<this->numPlace();
        AbuleduLabel *etiqCastee = static_cast<AbuleduLabel *>(event->source());
        //qDebug()<<"Numero etiquette survolee"<<this->numPlace()<<" hauteur"<<this->height();
        //qDebug()<<"Numero de la source"<<etiqCastee->numPlace()<<"taille etiquette"<<etiqCastee->height();

        qDebug()<<pixel::isModeVertical();

        int place=0;
        switch(etiqCastee->numPlace() - this->numPlace())
        {
        case 1: //On monte d'une étiquette exactement
            if(pixel::isModeVertical())
            {
                //qDebug() <<" on bouge l'etiquette en place "<< etiqCastee->numPlace();
                this->move(etiqCastee->lastPos().x(),etiqCastee->lastPos().y());
                //qDebug() << "case 1";
                // On échange les numéros de place
                place = this->numPlace();
                this->setNumPlace(etiqCastee->numPlace());
                etiqCastee->setNumPlace(place);
                //qDebug() << m_margeVerticale;
                etiqCastee->setLastPos(QPoint(this->pos().x(),this->pos().y()-this->height()-m_margeVerticale)); // On affecte les nouvelles positions
                this->setLastPos(this->pos());
            }
            else
            {
                //qDebug() <<" on bouge l'etiquette en place "<< etiqCastee->numPlace();
                this->move(etiqCastee->lastPos().x(),etiqCastee->lastPos().y());
                //qDebug() << "case 1";
                // On échange les numéros de place
                place = this->numPlace();
                this->setNumPlace(etiqCastee->numPlace());
                etiqCastee->setNumPlace(place);
                //qDebug() << m_margeVerticale;
                etiqCastee->setLastPos(QPoint(this->pos().x() -this->width() -m_margeHorizontale,this->pos().y())); // On affecte les nouvelles positions
                this->setLastPos(this->pos());
            }

            break;
        case -1: // On descend d'une étiquette exactement
            //qDebug() <<" on bouge l'etiquette en place "<< etiqCastee->numPlace();
            if(pixel::isModeVertical())
            {
                this->move(etiqCastee->lastPos().x(),etiqCastee->lastPos().y());
                //qDebug() << "case -1";
                place = this->numPlace();
                this->setNumPlace(etiqCastee->numPlace());
                etiqCastee->setNumPlace(place);
                etiqCastee->setLastPos(QPoint(this->pos().x(),this->pos().y()+this->height()+m_margeVerticale)); // On affecte les nouvelles positions
                this->setLastPos(this->pos());
                break;
            }
            else
            {
                //qDebug() <<" on bouge l'etiquette en place "<< etiqCastee->numPlace();
                this->move(etiqCastee->lastPos().x(),etiqCastee->lastPos().y());
                //qDebug() << "case 1";
                // On échange les numéros de place
                place = this->numPlace();
                this->setNumPlace(etiqCastee->numPlace());
                etiqCastee->setNumPlace(place);
                //qDebug() << m_margeVerticale;
                etiqCastee->setLastPos(QPoint(this->pos().x() +this->width() +m_margeHorizontale,this->pos().y())); // On affecte les nouvelles positions
                this->setLastPos(this->pos());
            }

        }
    }
}

void AbuleduLabel::mouseMoveEvent(QMouseEvent *event)
{
    event->accept();
}

//Petite astuce, quand le "UI" place les objets ça provoque l'evenement et donc on recupere les infos ... (à vérifier que ça ne créé pas d'effets de bords ?)
void AbuleduLabel::moveEvent(QMoveEvent *event)
{
    event->accept();
    //qDebug() << "Deplace l'objet " << this->objectName();
    setLastPos(pos());
}
