/**
  * Aller (le terrier d'AbulEdu)
  *
  * @warning aucun traitement d'erreur n'est pour l'instant implémenté
  * @see https://redmine.ryxeo.com/projects/leterrier-aller
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
  */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef ABULEDULABEL_H
#define ABULEDULABEL_H

#include <QLabel>
#include <QObject>
#include <QDrag>
#include <QMouseEvent>

/** Cette classe hérite de QLabel et permet de gérer une liste de AbuleduLabel's */

class AbuleduLabel : public QLabel
{
    Q_OBJECT
    Q_PROPERTY(int intOrdre READ numOrdre WRITE setNumOrdre DESIGNABLE true)
    Q_PROPERTY(int intPlace READ numPlace WRITE setNumPlace)
    Q_PROPERTY(QPoint lastPos READ lastPos WRITE setLastPos)
public:
    explicit AbuleduLabel(QWidget *parent = 0);
    explicit AbuleduLabel(QString texte, int numOrdre = -1, int numPlace = -1, QWidget *parent = 0);
    /** Attribue un numéro d'ordre de classement à l'étiquette
      * i.e. ordre alphabétique
      * @param intOrdre le numéro d'ordre
      */
    void setNumOrdre(int intOrdre);

    /** Retourne le numéro d'ordre de classement de l'étiquette
      * i.e. ordre alphabétique
      * @return le numéro d'ordre -1 par défaut
      */
    int numOrdre();

    /** Attribue un rang physique à l'étiquette
      * @param intPlace le numéro d'ordre
      */
    void setNumPlace(int intPlace);

    /** Retourne le rang physique de l'étiquette
      * @return le numéro d'ordre -1 par défaut
      */
    int numPlace();

    /** Stocke la position de l'étiquette après son déplacement
      * @param lastPoint la position a mémoriser
      */
    void setLastPos(QPoint lastPos);

    /** Fonction surchargée
      * @param x,y abscisse et ordonnée de la position à mémoriser
      */
    void setLastPos(int x,int y);

    /** Retourne la position mémorisée
      * @return lastPoint la position mémorisée
      */
    QPoint lastPos();

    /** Fixe l'espace vertical entre deux étiquettes 10 par défaut
      * @param marge
      */
    void setMargeVerticale(int marge);

    /** Fixe l'espace horizontal entre deux étiquettes 10 par défaut
      * @param marge
      */
    void setMargeHorizontale(int marge);

    /** Retourne l'espace vertical entre deux étiquettes
      * @return
      */
    int margeVerticale();

    /** Retourne l'espace horizontal entre deux étiquettes
      * @return
      */
    int margeHorizontale();

    /** Pixelise l'image dans l'étiquette
      * @param taillePixel
      */
    void pixelize(int taillePixel);

private:
    /** Le numéro d'ordre dans le classement -1 par défaut */
    int m_intOrdre;

    /** Le rang dans la liste des étiquettes -1 par défaut */
    int m_intPlace;

    /** La position de l'étiquette (avant déplacement) */
    QPoint m_lastPos;

    /** Espace vertical entre deux étiquettes */
    int m_margeVerticale;

    /** Espace horizontal entre deux etiquettes */
    int m_margeHorizontale;
    /** Mime type de abuleduLabel */
    QString m_mimecode;
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void dragEnterEvent(QDragEnterEvent *);
signals:
    void abuleduLabelSelected(int rang);
public slots:

protected:
    void moveEvent(QMoveEvent *);
};

#endif // ABULEDULABEL_H
