/**
  * Classe 
  * @see https://redmine.ryxeo.com/projects/leterrier/leterrier-pixel
  * @author 2011 Gabriel Cadeillan <gabriel.cadeillan@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  *
  *
  */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#include "pixel.h"
#include "ui_pixel.h"
#include <QDebug>
#include <QCursor>
#include <QInputDialog>
#include <QMessageBox>
#include <QFileDialog>
#include <QDir>
#include <vector>
#include <sstream>
#include <QStringList>
#include <time.h>
#include "abuleduaproposv0.h"


bool pixel::m_modeVertical=false;

pixel::pixel(QWidget *parent) :
        AbulEduExerciceV0(parent),
        ui(new Ui::pixel)
{
    QRect ecran=QApplication::desktop()->screenGeometry();
    //qDebug()<<"Ecran : "<< ecran.width()<<" X "<<ecran.height();
    this->setGeometry(0,0,ecran.width(),ecran.height()-20);

    m_modeVertical = true;

    ui->setupUi(this);
    this->setGeometry(0,0,ecran.width(),ecran.height()-20);


    m_nombreImage=4;
    m_ecartHorizontalLabels=m_ecartVerticalLabels=0;

    m_dossierData.setPath("data");
    m_dossierExercices.setPath("data/exercices");

    if(!m_dossierExercices.exists())
        qDebug()<<m_dossierData.mkdir("exercices");

    ui->btnProposer->setEnabled(false);
    ui->btnStart->setEnabled(false);
    ui->btnChangerMode->setEnabled(false);


    ui->textConsigne->setText(trUtf8("Créez ou Ouvrez un exercice, puis cliquez sur Commencer"));

    AbulEduAproposV0 *monAide=new AbulEduAproposV0(this);
    monAide->accept();

    ui->actionCr_er_Exercice->setText(trUtf8("Créer exercice..."));


}

pixel::~pixel()
{
    delete ui;
}

void pixel::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void pixel::on_btnStart_clicked()
{
    bool ok;
    int difficulte = QInputDialog::getInteger(this, trUtf8("Difficulté"), trUtf8("Choisissez un niveau de difficulté"),1,1,3,1,&ok);

    if(ok && !m_exerciceChoisi.isEmpty() && !m_listeImages.isEmpty())
    {

        for(unsigned int i=0;i<m_vignettes.size();i++)
        {
            m_vignettes[i]->close();
        }
        m_vignettes.clear();

        for(int i=0;i<m_nombreImage*2;i++)
        {
            m_vignettes.push_back(new AbuleduLabel(this));
        }

        for(int i=0;i<m_nombreImage*2;i++)
        {
            m_vignettes[i]->setText("");
            m_vignettes[i]->setGeometry(0,0,160,120);

            if(i<m_nombreImage)
            {
                QString nomComplet = m_listeImages[i];
                m_vignettes[i]->setPixmap(QPixmap(nomComplet));
                m_vignettes[i]->setAcceptDrops(true);
                m_vignettes[i]->setNumOrdre(i+1);
                m_vignettes[i]->setNumPlace(i+1);
                m_vignettes[i]->setFrameShape(QFrame::Box);
                m_vignettes[i]->setCursor(Qt::PointingHandCursor);
            }
            else
            {
                QString nomComplet = m_listeImages[i-m_nombreImage];
                m_vignettes[i]->setPixmap(QPixmap(nomComplet));
                m_vignettes[i]->setNumOrdre(i+1-m_nombreImage);
                m_vignettes[i]->setNumPlace(i+1-m_nombreImage);
                m_vignettes[i]->setCursor(QCursor());
                m_vignettes[i]->pixelize(10*difficulte);
            }
            m_vignettes[i]->show();

        }

        melangerVignettes();
        ui->lblResultat->setPixmap(QPixmap());

        calculerEcarts();
        positionnerLabels();

        if(m_modeVertical)
            ui->textConsigne->setText(trUtf8("Clique sur une image à gauche et maintiens pour la faire glisser vers ses voisines. Essaie de la placer en face de son equivalent pixellisé."));
        else
            ui->textConsigne->setText(trUtf8("Clique sur une image en bas et maintiens pour la faire glisser vers ses voisines. Essaie de la placer en dessous de son equivalent pixellise."));

        ui->btnProposer->setEnabled(true);
        ui->btnStart->setEnabled(false);
        ui->btnChangerMode->setEnabled(true);
    }

}

void pixel::dragMoveEvent(QDragMoveEvent *event)
{
    event->acceptProposedAction();
}

void pixel::dropEvent(QDropEvent *event)
{
    event->acceptProposedAction();
}

void pixel::resizeEvent(QResizeEvent *event)
{
    int taille =  m_vignettes.size();
    qDebug() << "taille :" << taille;
    event->accept();
    calculerEcarts();
    positionnerBoutons();
    positionnerLabels();
    qDebug()<<this->width()<<"x"<<this->height();
}

void pixel::dragEnterEvent(QDragEnterEvent *event)
{
    if (event->mimeData()->hasFormat("text/plain"))
    {
        event->acceptProposedAction();
    }
}

void pixel::on_btnChangerMode_clicked()
{
    m_modeVertical=m_modeVertical^true;
    positionnerBoutons();
    positionnerLabels();
    if(m_modeVertical)
        ui->textConsigne->setText(trUtf8("Clique sur une image a droite et maintiens pour la faire glisser vers ses voisines. Essaie de la placer en face de son equivalent pixellise."));
    else
        ui->textConsigne->setText(trUtf8("Clique sur une image en bas et maintiens pour la faire glisser vers ses voisines. Essaie de la placer en dessous de son equivalent pixellise."));

}

void pixel::on_btnQuitter_clicked()
{
    this->close();
}

void pixel::on_btnProposer_clicked()
{
    int nombreErreur = 0;

    for(int i=m_nombreImage;i<m_nombreImage*2;i++)
    {
        if(m_vignettes[i]->numPlace() != m_vignettes[i-m_nombreImage]->numPlace())
        {
            nombreErreur++;
        }
        QString nomComplet = m_listeImages[m_vignettes[i]->numOrdre()-1];
        m_vignettes[i]->setPixmap(QPixmap(nomComplet));
    }

    if(nombreErreur==0)
    {
        ui->textConsigne->setText(trUtf8("Tu n'as fait aucune erreur ! Bravo ! Tu peux cliquer sur Commencer pour changer la difficulté."));
        ui->lblResultat->setPixmap(QPixmap("data/images/will-win.png"));
    }
    else
    {
        ui->textConsigne->setText(trUtf8("Dommage ! Tu as fait ") + QString::number(nombreErreur)+ trUtf8(" erreurs.Tu peut cliquer sur Commencer pour changer la difficulté."));
        ui->lblResultat->setPixmap(QPixmap("data/images/will-lose.png"));
    }

    for(int i=0;i<m_nombreImage;i++)
    {
        m_vignettes[i]->setAcceptDrops(false);
    }
    ui->btnProposer->setEnabled(false);
    ui->btnStart->setEnabled(true);
    ui->btnChangerMode->setEnabled(false);

}

void pixel::setNombreImage(int nombreImage)
{

    m_nombreImage = nombreImage;

}

void pixel::calculerEcarts()
{
    if(m_vignettes.size()!=0)
    {
        m_ecartHorizontalLabels=(this->width() -200 -m_nombreImage*180)/(m_nombreImage+1);
        m_ecartVerticalLabels=(this->height() -80 -m_nombreImage*135)/(m_nombreImage+1);

        if(m_ecartHorizontalLabels<0)
            m_ecartHorizontalLabels=0;
        if(m_ecartVerticalLabels<0)
            m_ecartVerticalLabels=0;

    qDebug() << m_ecartHorizontalLabels;

        for(int i=0;i<m_nombreImage;i++)
        {
            m_vignettes[i]->setMargeVerticale(m_ecartVerticalLabels);
            m_vignettes[i]->setMargeHorizontale(m_ecartHorizontalLabels);
        }
    }
}

void pixel::positionnerLabels()
{

    if(m_vignettes.size()!=0)
    {
        for(int i=0;i<m_nombreImage*2;i++)
        {
            if(m_modeVertical)
            {
                if(i<m_nombreImage)
                    m_vignettes[i]->move(160 + 2*m_ecartHorizontalLabels, (m_vignettes[i]->numPlace()-1)*120 + m_vignettes[i]->numPlace()*m_ecartVerticalLabels+70);
                else
                {
                    m_vignettes[i]->move(2*160 + 3*m_ecartHorizontalLabels, (m_vignettes[i]->numPlace()-1)*120 + m_vignettes[i]->numPlace()*m_ecartVerticalLabels+70);
                }
            }
            else
            {
                if(i<m_nombreImage)
                    m_vignettes[i]->move((m_vignettes[i]->numPlace()-1)*160 + m_vignettes[i]->numPlace()*m_ecartHorizontalLabels, 120 +2*m_ecartVerticalLabels+70);
                else
                    m_vignettes[i]->move((m_vignettes[i]->numPlace()-1)*160 + m_vignettes[i]->numPlace()*m_ecartHorizontalLabels, m_ecartVerticalLabels+70);

            }
        }
        ui->lblResultat->move(this->width()-150, this->height()-330);
    }
}

void pixel::positionnerBoutons()
{
    if(this->height()>280)
    {
        ui->btnStart->move(this->width()-175,this->height()-460);
        ui->btnChangerMode->move(this->width()-175,this->height()-400);
        ui->btnProposer->move(this->width()-175,this->height()-220);
        ui->btnQuitter->move(this->width()-175,this->height()-160);
    }
    //ui->textConsigne->setGeometry(10,10,this->width()-185,30);

}

bool pixel::isModeVertical()
{
    return m_modeVertical;
}

QString pixel::enregistrerImage(QString imagePath, QString dossierPath)
{
    QPixmap pixmap = QPixmap(imagePath);
    pixmap=pixmap.scaled(160,120,Qt::KeepAspectRatioByExpanding,Qt::FastTransformation);

    QString nomFichier = QString::fromStdString(imagePath.toStdString().substr(imagePath.toStdString().rfind('/')+1, imagePath.toStdString().rfind('.')-imagePath.toStdString().rfind('/')-1));

    //qDebug() << nomFichier;

    if(!pixmap.save(dossierPath+"/"+nomFichier+"pix.jpg"))
        QMessageBox::critical(this, "Erreur", trUtf8("Erreur à la création de l'image, création de l'exercice impossible"));

    return nomFichier;

}

void pixel::on_actionCr_er_Exercice_activated()
{
    bool ok = false;
    QString nomExercice = QInputDialog::getText(this, trUtf8("Créer un exercice"), trUtf8("Quel est le nom de l'exercice ?"), QLineEdit::Normal, QString(), &ok);

    if (ok)
    {
        QDir dossierExercice;
        dossierExercice.setPath("data/exercices/"+nomExercice);

        if(!nomExercice.isEmpty() && !dossierExercice.exists())
        {
            qDebug() << "Creation d'un exo : "<<m_dossierExercices.mkdir(nomExercice);
            QFile file("data/exercices/"+nomExercice+"/"+nomExercice+".txt");
            // On ouvre notre fichier en lecture seule et on verifie l'ouverture
            if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
                return;

            // Creation d'un objet QTextStream à partir de notre objet QFile
            QTextStream flux(&file);
            // On choisit le codec correspondant au jeu de caractère que l'on souhaite ; ici, UTF-8
            flux.setCodec("UTF-8");


            bool ok;
            int nombreImages = QInputDialog::getInteger(this, "Nombre d'images", "Choisissez un nombre d'images",4,3,6,1,&ok);

            if(ok)
            {
                for(int i=0;i<nombreImages;i++)
                {
                    QFileDialog fenetreChoixImage;
                    fenetreChoixImage.setViewMode(QFileDialog::List);
                    QString imageOrigine = fenetreChoixImage.getOpenFileName(this, "Ouvrir l'image "+QString::number(i+1), QString(), "Images (*.png *.gif *.jpg *.jpeg *.bmp *.svg)");
                    flux << "data/exercices/"+nomExercice+"/"+enregistrerImage(imageOrigine,"data/exercices/"+nomExercice)<<"pix.jpg"<<endl;
                    if(imageOrigine.isEmpty())
                    {
                        file.close();
                        supprimerExercice(nomExercice);
                        return;
                    }
                }

                QMessageBox::information(this, trUtf8("Création d'un exercice terminée"), trUtf8("Exercice ")+nomExercice+trUtf8(" créé avec succès. Vous pouvez maintenant l'ouvrir dans le menu Fichier->Ouvrir exercice"));
                return;
            }
            else
            {
                file.close();
                supprimerExercice(nomExercice);
                QMessageBox::critical(this, trUtf8("Création annulée"), trUtf8("Création de l'exercice annulée"));
                return;
            }
        }
        else
        {
            QMessageBox::critical(this, "ERREUR", trUtf8("Le nom de l'exercice est vide ou existe dejà !"));
        }
    }
}

void pixel::on_actionOuvrir_Exercice_activated()
{

    for(unsigned int i=0;i<m_vignettes.size();i++)
    {
        m_vignettes[i]->close();
    }
    m_vignettes.clear();
    m_listeImages.clear();

    QStringList listeExercices = m_dossierExercices.entryList(QDir::Dirs | QDir::NoDotAndDotDot,QDir::Name);

    if(listeExercices.isEmpty())
    {
        QMessageBox::critical(this, "ERREUR", trUtf8(" Aucun exercice trouvé ! \n Créez d'abord un exercice ou vérifiez que l'exercice se trouve bien dans le dossier exercices du programme"));
        return;
    }
    bool ok;
    m_exerciceChoisi = QInputDialog::getItem(this, "Ouvrir un exercice", "Choisissez un exercice", listeExercices,0,false,&ok);

    if(!ok || m_exerciceChoisi.isEmpty())
    {
        qDebug() << "Aucun exercice choisi";
    }
    else if(!QDir("data/exercices/"+m_exerciceChoisi).exists())
    {
        qDebug() << "Aucun exercice choisi";
    }
    else
    {
        QFile fichierTexteExercice("data/exercices/"+m_exerciceChoisi+"/"+m_exerciceChoisi+".txt");
        fichierTexteExercice.open(QIODevice::ReadOnly | QIODevice::Text);
        QTextStream flux(&fichierTexteExercice);
        QString ligne;

        while(!flux.atEnd())
        {
            ligne = flux.readLine();
            //traitement de la ligne
            qDebug() << "ligne : "<<ligne;
            m_listeImages<<ligne;
        }
        qDebug()<< "listesize :"<< m_listeImages.size();
        m_nombreImage=m_listeImages.size();
        ui->btnStart->setEnabled(true);
        ui->textConsigne->setText("L'exercice \""+m_exerciceChoisi+"\" est ouvert. Clique sur Commencer pour démarrer l'exercice.");

        fichierTexteExercice.close();
    }


}

void pixel::supprimerExercice(QString nomExercice)
{
    qDebug() << "rvmdir :" << m_dossierExercices.path()+"/"+nomExercice;
    if(m_dossierExercices.exists() && QDir(m_dossierExercices.path()+"/"+nomExercice).exists())
    {
        qDebug() << removeDir(m_dossierExercices.path()+"/"+nomExercice);
    }
}

bool pixel::removeDir(QString dirPath)
{
    QDir folder(dirPath);
    folder.setFilter(QDir::NoDotAndDotDot | QDir::AllEntries);
    foreach(QFileInfo fileInfo, folder.entryInfoList())
    {
        if(fileInfo.isDir())
        {
            if(!removeDir(fileInfo.filePath()))
                return false;
        }
        else if(fileInfo.isFile())
        {
            if(!QFile::remove(fileInfo.filePath()))
            {
                qDebug() << "Unable to remove file : " << fileInfo.filePath();
                return false;
            }
        }
    }

    if(!folder.rmdir(folder.absolutePath()))
    {
        qDebug() << "Unable to remove folder : " << dirPath << ". Maybe this folder is not empty";
        return false;
    }

    return true;
}

void pixel::melangerVignettes()
{
    srand (time (NULL));
    int r=0;

    QList<int> liste;
    for(int i=0;i<m_nombreImage;i++)
        liste.append(i);

    int j=m_nombreImage;

    while(j<m_nombreImage*2)
    {
        r=rand()%liste.size();
        /*qDebug()<< "taille liste :"<< liste.size();
        qDebug()<< "r choisi :" << liste.at(r);
        qDebug()<<liste;*/
        if(m_vignettes[j]->numPlace() != liste.at(r)+1 || j==m_nombreImage*2-1)
        {
            m_vignettes[j]->setNumPlace(liste.at(r)+1);
            //qDebug()<<"La vignette "<<j<<" prend la place "<<liste.at(r)+1;
            //qDebug()<<liste;
            liste.removeAt(r);
            //qDebug()<<liste;
            j++;
        }
    }

}
