/**
  * Classe pixel
  * @see https://redmine.ryxeo.com/projects/leterrier/leterrier-pixel
  * @author 2011 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  *
  *
  */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef PIXEL_H
#define PIXEL_H

#include <QMainWindow>
#include <QDesktopWidget>
#include "lib/abuledulabel/abuledulabel.h"
#include <vector>
#include <QDir>
#include "abuleduexercicev0.h"


namespace Ui {
    class pixel;
}

class pixel : public AbulEduExerciceV0 {
    Q_OBJECT
public:
    pixel(QWidget *parent = 0);
    ~pixel();

    /** Change le nombre d'images dans le programme (m_nombreImage)
      * @param nomberImage
      */

    void setNombreImage(int nombreImage);

    /** Calcule et mets à jour les écart (marges verticales et horizontales) des AbuleduLabels pour les positionner convenablement lors des resizeEvent
      * @return
      */
    void calculerEcarts();

    /** Change les positions des AbuleduLabels en fonction des marges et du m_modeVertical
      * @return
      */
    void positionnerLabels();

    /** Change les positions des boutons en fonction dela taille de la fenêtre
      * @return
      */
    void positionnerBoutons();

    /** Retourne vrai si le mode vertical est actif
      * @return m_modeVertical
      */
    static bool isModeVertical();

    /** Enregistre une image copie de celle en parametre imagePath, mais scaled et mise aux bonnes dimmensions (160*120). Elle est copié dans le dossier dossierPath. Retourne ensuite le nom du fichier créé (pour l'ajouter à la liste).
      * @param imagePath
      * @param dossierPath
      * @return nomFichier
      */
    QString enregistrerImage(QString imagePath, QString dossierPath);

protected:
    void changeEvent(QEvent *e);

private:
    Ui::pixel *ui;
    void dragMoveEvent(QDragMoveEvent *event);
    void dropEvent(QDropEvent *event);
    void dragEnterEvent(QDragEnterEvent *event);
    void resizeEvent(QResizeEvent *event);

    /** Le nombre d'images dans l'exercice actif*/
    int m_nombreImage;

    /** L'écart (marge) vertical des AbuleduLabels ouverts*/
    int m_ecartVerticalLabels;

     /** L'écart (marge) horizontal des AbuleduLabels ouverts*/
    int m_ecartHorizontalLabels;

    /** Booléen indiquant si le mode Vertical est actif (si faux c'est le mode horizontal)*/
    static bool m_modeVertical;

    /** Vector des AbuleduLabels de l'exercice ouvert*/
    std::vector<AbuleduLabel*> m_vignettes;

    /** Dossier des data*/
    QDir m_dossierData;

    /** Dossier des exercices (data/exercices)*/
    QDir m_dossierExercices;

    /** Liste des chemins des images de l'exercice ouvert*/
    QStringList m_listeImages;

    /** Nom de l'exercice ouvert*/
    QString m_exerciceChoisi;



private slots:
    void on_actionCr_er_Exercice_activated();
    void on_btnProposer_clicked();
    void on_btnQuitter_clicked();
    void on_btnChangerMode_clicked();
    void on_btnStart_clicked();
    void on_actionOuvrir_Exercice_activated();

    /** Supprimme l'exercice dont le nom est en parametre. Retourne true si l'opération a réussi.(utilisation de removeDir)
      @param nomExercice
      */
    void supprimerExercice(QString nomExercice);

    /** Supprimme le dossier dont le path est en parametre, et ce qu'il contient. Retourne true si l'opération a réussi.
      @param dirPath
      */
    bool removeDir(QString dirPath);

    /** Mélange les vignettes pixellisées*/
    void melangerVignettes();

};

#endif // PIXEL_H
